import sys

import numpy as np
import matplotlib.pyplot as plt

from scipy.integrate import cumulative_trapezoid
from scipy.optimize import curve_fit, fsolve
from scipy.stats import linregress

from nmreval.data import Points


# Read data
dataName = sys.argv[1]
try:
    data = np.loadtxt(dataName, skiprows=0).T
    temp = data[0]
    heat_capacity = data[1]
except IndexError:
    exit()

print(np.round(temp, decimals=3)[:30])


def get_fictive_temperature(pts: Points, glass: tuple[float, float], liquid: tuple[float, float]):
    min_glass, max_glass = min(glass), max(glass)
    min_liquid, max_liquid = min(liquid), max(liquid)

    region = pts.copy()
    region.cut(min_glass, max_liquid)

    glass_regime = (min_glass < region.x) & (region.x < max_glass)
    regress = linregress(region.x[glass_regime], region.y[glass_regime])
    glass_extrapolation = regress.slope * region.x + regress.intercept

    region.y -= glass_extrapolation

    liquid_regime = (min_liquid < region.x) & (region.x < max_liquid)
    regress2 = linregress(region.x[liquid_regime], region.y[liquid_regime])
    liquid_extrapolation = regress2.slope * region.x + regress2.intercept

    real_area = -cumulative_trapezoid(region.y, region.x, initial=0)
    real_area -= real_area[-1]
    equivalent_area = cumulative_trapezoid(liquid_extrapolation, region.x, initial=0)
    equivalent_area -= equivalent_area[-1]
    equivalent_area *= -1

    return region.x, np.round(region.x[np.argmin(np.abs(real_area[:, None] - equivalent_area[None, :]), axis=1)], decimals=4)


curve = Points(x=temp, y=heat_capacity, name=dataName)
# plt.plot(curve.x, curve.y)
# plt.show()

# First step: Calculate fictive Temperature T_f(T) from Cp data, then dT_f/dT

temperatures, fictiveTemp = get_fictive_temperature(curve, glass=(150, 160), liquid=(190, 200))

# Determine limiting T_f, i.e. T_f at lowest temperature
initial_Tf = np.mean(fictiveTemp[:20])

rate = curve.value

# Calculate dTf/dT
# with np.errstate(all='ignore'):
#     dTfdT = np.diff(fictiveTemp)/np.diff(temperatures)  #  np.gradient(fictiveTemp, temperatures)
# nan_filter = ~np.isnan(dTfdT)
# temperatures = 0.5 * (temperatures[:-1] + temperatures[1:])[nan_filter]
# # fictiveTemp = fictiveTemp[nan_filter]
# dTfdT = dTfdT[nan_filter]

print(fictiveTemp[:30])

R = 8.314


def tnm_function(temperature, tau_g, x, beta, energy, tg):
    step = len(temperature)

    Tf = np.empty(step)
    Tf[0] = temperature[0]

    delT = np.diff(temperature)
    delt = np.abs(delT) * 60 / rate

    tau = np.empty(step)
    dttau = np.zeros(step)
    temp_0 = temperature[0]

    for i in range(0, step-1):
        tau[i] = relax(temperature[i+1], Tf[i], tau_g, tg, energy, x)
        dttau[:i] += delt[i] / tau[i]
        Tf[i+1] = np.sum(delT[:i] * (1-np.exp(-dttau[:i]**beta))) + temp_0

    return Tf


def relax(t, tf, tau_g, t_glass, ea, x):
    h = ea/R
    return tau_g * np.exp((x*h / t) + ((1 - x) * h / tf) - h / t_glass)


# Use T_g = T_fictive for tau determination
T_g = initial_Tf

# Start values
p0 = [10, 0.5, 0.4, 225085]


def tnmh_fit(tg):
    def wrap(x, tau_0, xx, beta, energy):
        modelTemp = np.r_[x[::-1], x[1:]]

        TNMH = tnm_function(modelTemp, tau_0, xx, beta, energy, tg)
        res = np.gradient(TNMH, modelTemp)

        return res[len(x)-1:]

    return wrap


# Use only every 20th data point to reduce fitting time
temperaturesInterpol = np.linspace(temperatures[0], temperatures[-1], 200)
dTfdTInterpol = np.interp(temperaturesInterpol, temperatures, dTfdT)

res = curve_fit(tnmh_fit(T_g), temperaturesInterpol, dTfdTInterpol, p0)


plt.plot(temperatures, dTfdT, label='dTf/dT')
plt.plot(temperaturesInterpol, tnmh_fit(T_g)(temperaturesInterpol, *res[0]), label='fit')
plt.legend()
plt.show()
